@@ -3,6 +3,7 @@ from __future__ import annotations
 __all__ = ("ITerm2Image",)
 
 import io
+import os
 import re
 import sys
 import warnings
@@ -14,8 +15,23 @@ from typing import Any, Dict, Optional, Set, Tuple, Union
 import PIL
 
 from ..exceptions import TermImageWarning, _style_error
-from ..utils import CSI, ESC, OSC, ST, get_terminal_name_version, get_terminal_size
+from ..utils import (
+    CSI,
+    OSC,
+    ST,
+    ClassInstanceProperty,
+    ClassProperty,
+    get_terminal_name_version,
+    get_terminal_size,
+    write_tty,
+)
 from .common import GraphicsImage, ImageSource
+from .kitty import (
+    DELETE_ALL_IMAGES,
+    DELETE_CURSOR_IMAGES,
+    DELETE_ALL_IMAGES_b,
+    DELETE_CURSOR_IMAGES_b,
+)
 
 # Constants for render methods
 LINES = "lines"
@@ -27,7 +43,9 @@ class ITerm2Image(GraphicsImage):
 
     See :py:class:`GraphicsImage` for the complete description of the constructor.
 
-    **Render Methods:**
+    |
+
+    **Render Methods**
 
     :py:class:`ITerm2Image` provides two methods of :term:`rendering` images, namely:
 
@@ -37,13 +55,13 @@ class ITerm2Image(GraphicsImage):
 
        Pros:
 
-         * Good for use cases where it might be required to trim some lines of the
-           image.
+       * Good for use cases where it might be required to trim some lines of the
+         image.
 
        Cons:
 
-         * Image drawing is very slow on iTerm2 due to the terminal emulator's
-           performance.
+       * Image drawing is very slow on iTerm2 due to the terminal emulator's
+         performance.
 
     WHOLE
        Renders an image all at once i.e the entire image data is encoded into one
@@ -52,15 +70,15 @@ class ITerm2Image(GraphicsImage):
 
        Pros:
 
-         * Render results are more compact (i.e less in character count) than with
-           the ``lines`` method since the entire image is encoded at once.
-         * Image drawing is faster than with ``lines`` on most terminals.
-         * Smoother animations.
+       * Render results are more compact (i.e less in character count) than with
+         the **LINES** method since the entire image is encoded at once.
+       * Image drawing is faster than with **LINES** on most terminals.
+       * Smoother animations.
 
        Cons:
 
-          * This method currently doesn't work well on iTerm2 and WezTerm when the image
-            height is greater than the terminal height.
+       * This method currently doesn't work well on iTerm2 and WezTerm when the image
+         height is greater than the terminal height.
 
     NOTE:
         The **LINES** method is the default only because it works properly in all cases,
@@ -71,6 +89,52 @@ class ITerm2Image(GraphicsImage):
     :py:meth:`set_render_method() <BaseImage.set_render_method>` using the names
     specified above.
 
+    |
+
+    **Style-Specific Render Parameters**
+
+    See :py:meth:`BaseImage.draw` (particularly the *style* parameter).
+
+    * **method** (*None | str*) → Render method override.
+
+      * ``None`` → the current effective render method of the instance is used.
+      * *default* → ``None``
+
+    * **mix** (*bool*) → Cell content inter-mix policy (**Only supported on WezTerm**,
+      ignored otherwise).
+
+      * ``False`` → existing contents of cells within the region covered by
+        the drawn render output are erased
+      * ``True`` → existing cell contents show under transparent areas of the
+        drawn render output
+      * *default* → ``False``
+
+    * **compress** (*int*) → ZLIB compression level, for renders re-encoded in PNG
+      format.
+
+      * ``0`` <= *compress* <= ``9``
+      * ``1`` → best speed, ``9`` → best compression, ``0`` → no compression
+      * *default* → ``4``
+      * Results in a trade-off between render time and data size/draw speed
+
+    * **native** (*bool*) → Native animation policy. [1]_
+
+      * ``True`` → use the protocol's native animation feature
+      * ``False`` → use the normal animation
+      * *default* → ``False``
+      * *alpha*, *repeat*, *cached* and *style* do not apply
+      * Ignored if the image is not animated or *animate* is ``False``
+      * Normal restrictions for sizing of animations do not apply
+      * Uses **WHOLE** render method
+      * The terminal emulator completely controls the animation
+
+    * **stall_native** (*bool*) → Native animation execution control.
+
+      * ``True`` → block until ``SIGINT`` (Ctrl+C) is recieved
+      * ``False`` → return as soon as the image is transmitted
+      * *default* → ``True``
+
+    |
 
     **Format Specification**
 
@@ -78,86 +142,59 @@ class ITerm2Image(GraphicsImage):
 
     ::
 
-        [method] [ m {0 | 1} ] [ c {0-9} ]
+        [ <method> ]  [ m <mix> ]  [ c <compress> ]
 
-    * ``method``: Render method override.
+    * ``method`` → render method override
 
-      Can be one of:
+      * ``L`` → **LINES** render method (current frame only, for animated images)
+      * ``W`` → **WHOLE** render method (current frame only, for animated images)
+      * ``N`` → Native animation [1]_ (ignored when used with non-animated images or
+        :py:class:`~term_image.image.ImageIterator`)
+      * *default* → current effective render method of the instance
 
-        * ``L``: **LINES** render method (current frame only, for animated images).
-        * ``W``: **WHOLE** render method (current frame only, for animated images).
-        * ``N``: Native animation. Ignored when used with non-animated images, WEBP
-          images or ``ImageIterator``.
+    * ``m`` → cell content inter-mix policy (**Only supported in WezTerm**, ignored
+      otherwise)
 
-      Default: Current effective render method of the image.
+      * ``mix`` → inter-mix policy
 
-    * ``m``: Cell content inter-mix policy (**Only supported in WezTerm**, ignored
-      otherwise).
+        * ``0`` → existing contents of cells in the region covered by the drawn
+          render output will be erased
+        * ``1`` → existing cell contents show under transparent areas of the drawn
+          render output
 
-      * If the character after ``m`` is:
+      * *default* → ``m0``
+      * e.g ``m0``, ``m1``
 
-        * ``0``, contents of cells in the region covered by the image will be erased.
-        * ``1``, the opposite, thereby allowing existing cell contents to show under
-          transparent areas of the image.
+    * ``c`` → ZLIB compression level, for renders re-encoded in PNG format
 
-      * If *absent*, defaults to ``m0``.
-      * e.g ``m0``, ``m1``.
+      * ``compress`` → compression level
 
-    * ``c``: ZLIB compression level, for images re-encoded in PNG format.
+        * An integer in the range ``0`` <= ``x`` <= ``9``
+        * ``1`` → best speed, ``9`` → best compression, ``0`` → no compression
 
-      * 1 -> best speed, 9 -> best compression, 0 -> no compression.
-      * This results in a trade-off between render time and data size/draw speed.
-      * If *absent*, defaults to ``c4``.
-      * e.g ``c0``, ``c9``.
+      * *default* → ``c4``
+      * e.g ``c0``, ``c9``
+      * Results in a trade-off between render time and data size/draw speed
 
+    |
 
-    ATTENTION:
-        Currently supported terminal emulators include:
+    IMPORTANT:
+        Currently supported terminal emulators are:
 
-          * `iTerm2 <https://iterm2.com>`_
-          * `Konsole <https://konsole.kde.org>`_ >= 22.04.0
-          * `WezTerm <https://wezfurlong.org/wezterm/>`_
+        * `iTerm2 <https://iterm2.com>`_
+        * `Konsole <https://konsole.kde.org>`_ >= 22.04.0
+        * `WezTerm <https://wezfurlong.org/wezterm/>`_
+
+    .. [1] Native animation support:
+
+       * Not all animated image formats may be supported by every supported terminal
+         emulator
+       * Not all supported terminal emulators implement this feature of the protocol
+         e.g on Konsole, the first frame is drawn but the image is not animated
+
+    |
     """
 
-    #: * ``x < 0``, JPEG encoding is disabled.
-    #: * ``0 <= x <= 95``, JPEG encoding is used, with the specified quality, for
-    #:   **most** non-transparent renders (at the cost of image quality).
-    #:
-    #: Only applies when not reading directly from file.
-    #:
-    #: By default, images are encoded in the PNG format (when not reading directly
-    #: from file) but in some cases, higher compression might be desired.
-    #: Also, JPEG encoding is significantly faster and can be useful to improve
-    #: non-native animation performance.
-    #:
-    #: .. hint:: The transparency status of some images can not be correctly determined
-    #:   in an efficient way at render time. To ensure the JPEG format is always used
-    #:   for a re-encoded render, disable transparency or set a background color.
-    JPEG_QUALITY: int = -1
-
-    #: Maximum size (in bytes) of image data for native animation.
-    #:
-    #: | :py:class:`TermImageWarning<term_image.TermImageWarning>` is issued
-    #:   (and shown **only the first time**, except a filter is set to do otherwise)
-    #:   if the image data size for a native animation is above this value.
-    #: | This value can be altered but should be done with caution to avoid excessive
-    #:   memory usage.
-    NATIVE_ANIM_MAXSIZE: int = 2 * 2**20  # 2 MiB
-
-    #: * ``True``, image data is read directly from file when possible and no image
-    #:   manipulation is required.
-    #: * ``False``, images are always loaded and re-encoded, in the PNG format by
-    #:   default.
-    #:
-    #: This is an optimization to reduce render times and is only applicable to the
-    #: **WHOLE** render method, since the the **LINES** method inherently requires
-    #: image manipulation.
-    #:
-    #: .. note:: This setting does not affect animations, native animations are always
-    #:   read from file when possible and frames of non-native animations have to be
-    #:   loaded and re-encoded.
-    READ_FROM_FILE: bool = True
-
     _FORMAT_SPEC: Tuple[re.Pattern] = tuple(
         map(re.compile, "[LWN] m[01] c[0-9]".split(" "))
     )
@@ -216,89 +253,240 @@ class ITerm2Image(GraphicsImage):
     _TERM: str = ""
     _TERM_VERSION: str = ""
 
+    jpeg_quality = ClassInstanceProperty(
+        lambda self_or_cls: getattr(self_or_cls, "_jpeg_quality", -1),
+        doc="""
+        JPEG encoding quality
+
+        :type: int
+
+        GET:
+            Returns the effective JPEG encoding quality of the invoker
+            (negative if disabled).
+
+        SET:
+            If invoked via:
+
+            * a **class**, the **class-wide** quality is set.
+            * an **instance**, the **instance-specific** quality is set.
+
+        DELETE:
+            If invoked via:
+
+            * a **class**, the **class-wide** quality is unset.
+            * an **instance**, the **instance-specific** quality is unset.
+
+        If:
+
+        * *value* < ``0``; JPEG encoding is disabled.
+        * ``0`` <= *value* <= ``95``; JPEG encoding is enabled with the given quality.
+
+        If **unset** for:
+
+        * a **class**, it uses that of its parent *iterm2* style class (if any) or the
+          default (disabled), if unset for all parents or the class has no parent
+          *iterm2* style class.
+        * an **instance**, it uses that of it's class.
+
+        By **default**, the quality is **unset** i.e JPEG encoding is **disabled** and
+        images are encoded in the PNG format (when not reading directly from file) but
+        in some cases, higher and/or faster compression may be desired.
+        JPEG encoding is significantly faster than PNG encoding and produces smaller
+        (in data size) output but **at the cost of image quality**.
+
+        NOTE:
+            * This property is :term:`descendant`.
+            * This optimization applies to only **re-encoded** (i.e not read directly
+              from file) **non-transparent** renders.
+
+        TIP:
+            The transparency status of some images can not be correctly determined
+            in an efficient way at render time. To ensure JPEG encoding is always used
+            for a re-encoded render, disable transparency or set a background color.
+
+            Furthermore, to ensure that renders with the **WHOLE** :term:`render method`
+            are always re-encoded, disable :py:attr:`read_from_file`.
+
+            This optimization is useful in improving non-native animation performance.
+
+        SEE ALSO:
+            * the *alpha* parameter of :py:meth:`~term_image.image.BaseImage.draw`
+              and the ``#``, ``bgcolor`` fields of the :ref:`format-spec`
+            * :py:attr:`read_from_file`
+        """,
+    )
+
+    @jpeg_quality.setter
+    def jpeg_quality(self_or_cls, quality: int) -> None:
+        if not isinstance(quality, int):
+            raise TypeError(
+                f"Invalid type for 'quality' (got: {type(quality).__name__})"
+            )
+        if quality > 95:
+            raise ValueError(f"'quality' out of range (got: {quality})")
+
+        self_or_cls._jpeg_quality = quality
+
+    @jpeg_quality.deleter
+    def jpeg_quality(self_or_cls) -> None:
+        try:
+            del self_or_cls._jpeg_quality
+        except AttributeError:
+            pass
+
+    native_anim_max_bytes = ClassProperty(
+        # 2 MiB default
+        lambda cls: getattr(__class__, "_native_anim_max_bytes", 2 * 2**20),
+        doc="""
+        Maximum size (in bytes) of image data for native animation
+
+        :type: int
+
+        GET:
+            Returns the set value.
+
+        SET:
+            A positive integer; the value is set on the *iterm2* render style baseclass
+            (:py:class:`ITerm2Image`).
+
+        DELETE:
+            The value is unset, thereby resetting it to the default.
+
+        :py:class:`~term_image.exceptions.TermImageWarning` is issued (and shown
+        **only the first time**, except a filter is set to do otherwise) if the
+        image data size for a native animation is above this value.
+
+        NOTE:
+            This property is :term:`descendant` but is always unset for all subclasses
+            and instances. Hence, setting/resetting it on this class, a subclass or an
+            instance affects this class, all its subclasses and all their instances.
+
+        WARNING:
+            This property should be altered with caution to avoid excessive memory
+            usage.
+        """,
+    )
+
+    @native_anim_max_bytes.setter
+    def native_anim_max_bytes(cls, max_bytes: int):
+        if not isinstance(max_bytes, int):
+            raise TypeError(
+                f"Invalid type for 'max_bytes' (got: {type(max_bytes).__name__})"
+            )
+        if max_bytes <= 0:
+            raise ValueError(f"'max_bytes' out of range (got: {max_bytes})")
+
+        __class__._native_anim_max_bytes = max_bytes
+
+    @native_anim_max_bytes.deleter
+    def native_anim_max_bytes(cls):
+        try:
+            del __class__._native_anim_max_bytes
+        except AttributeError:
+            pass
+
+    read_from_file = ClassInstanceProperty(
+        lambda self_or_cls: getattr(self_or_cls, "_read_from_file", True),
+        doc="""
+        Read-from-file optimization policy
+
+        :type: bool
+
+        GET:
+            Returns the effective read-from-file policy of the invoker.
+
+        SET:
+            If invoked via:
+
+            * a **class**, the **class-wide** policy is set.
+            * an **instance**, the **instance-specific** policy is set.
+
+        DELETE:
+            If invoked via:
+
+            * a **class**, the **class-wide** policy is unset.
+            * an **instance**, the **instance-specific** policy is unset.
+
+        If the value is:
+
+        * ``True``, image data is read directly from file when possible and no image
+          manipulation is required.
+        * ``False``, images are always re-encoded (in the PNG format by default).
+
+        If **unset** for:
+
+        * a **class**, it uses that of its parent *iterm2* style class (if any) or the
+          default (``True``), if unset for all parents or the class has no parent
+          *iterm2* style class.
+        * an **instance**, it uses that of it's class.
+
+        By **default**, the policy is **unset**, which is equivalent to ``True``
+        i.e the optimization is **enabled**.
+
+        NOTE:
+            * This property is :term:`descendant`.
+            * This is an optimization to reduce render times and is only applicable to
+              the **WHOLE** render method, since the the **LINES** method inherently
+              requires image manipulation.
+            * This property does not affect animations. Native animations are always
+              read from file when possible and frames of non-native animations have
+              to be re-encoded.
+
+        SEE ALSO:
+            :py:attr:`jpeg_quality`
+        """,
+    )
+
+    @read_from_file.setter
+    def read_from_file(self_or_cls, policy: bool) -> None:
+        if not isinstance(policy, bool):
+            raise TypeError(f"Invalid type for 'policy' (got: {type(policy).__name__})")
+
+        self_or_cls._read_from_file = policy
+
+    @read_from_file.deleter
+    def read_from_file(self_or_cls) -> None:
+        try:
+            del self_or_cls._read_from_file
+        except AttributeError:
+            pass
+
     @classmethod
-    def clear(cls) -> None:
-        """Clears all images on-screen.
-
-        Required and works only on Konsole, as text doesn't overwrite images.
-        """
-        if cls._TERM == "konsole":
-            # Seems Konsole utilizes the same image rendering implementation as it
-            # uses for the kiity graphics protocol.
-            _stdout_write(DELETE_ALL_IMAGES)
-
-    def draw(
-        self,
-        *args,
-        method: Optional[str] = None,
-        mix: bool = False,
-        compress: int = 4,
-        native: bool = False,
-        stall_native: bool = True,
-        **kwargs,
-    ):
-        """Draws an image to standard output.
-
-        Extends the common interface with style-specific parameters.
+    def clear(cls, cursor: bool = False, now: bool = False) -> None:
+        """Clears images.
 
         Args:
-            args: Positional arguments passed up the inheritance chain.
-            method: Render method override. If ``None`` or not given, the current
-              effective render method of the instance is used.
-            mix: Cell content inter-mix policy (**Only supported in WezTerm**, ignored
-              otherwise). If:
+            cursor: If ``True``, all images intersecting with the current cursor
+              position are cleared. Otherwise, all visible images are cleared.
+            now: If ``True`` the images are cleared immediately, without affecting
+              any standard I/O stream.
+              Otherwise they're cleared when next :py:data:`sys.stdout` is flushed.
 
-              * ``False``, contents of cells within the region covered by the image are
-                erased.
-              * ``True``, the opposite, thereby allowing existing text or image pixels
-                to show under transparent areas of the image.
-
-            compress: ZLIB compression level, for images re-encoded in PNG format.
-
-              An integer between 0 and 9: 1 -> best speed, 9 -> best compression, 0 ->
-              no compression. This results in a trade-off between render time and data
-              size/draw speed.
-
-            native: If ``True``, use native animation (if supported).
-
-              * Ignored for non-animations.
-              * *animate* must be ``True``.
-              * *alpha*, *repeat*, *cached* and *style* do not apply.
-              * Always loops infinitely.
-              * No control over frame duration.
-              * Not all animated image formats are supported e.g WEBP.
-              * The limitations of the **WHOLE** render method also apply.
-              * Normal restrictions for rendered/padding height of animations do not
-                apply.
-
-            stall_native: Native animation execution control. If:
-
-              * ``True``, block until ``SIGINT`` (Ctrl+C) is recieved.
-              * ``False``, return as soon as the image is transmitted.
-
-            kwargs: Keyword arguments passed up the inheritance chain.
-
-        Raises:
-            term_image.exceptions.ITerm2ImageError: Native animation is not supported.
-
-        See the ``draw()`` method of the parent classes for full details, including the
-        description of other parameters.
+        NOTE:
+            Required and works only on Konsole, as text doesn't overwrite images.
         """
-        if not (self._is_animated and kwargs.get("animate", True)):
-            # Prevent the arguments from being passed on
-            native = False
-            stall_native = True
+        # There's no point checking for forced support since this is only required on
+        # konsole which supports the protocol.
+        # `is_supported()` is first called to ensure `_TERM` has been set.
+        if cls.is_supported() and cls._TERM == "konsole":
+            # Konsole utilizes the same image rendering implementation as it
+            # uses for the kiity graphics protocol.
+            (write_tty if now else _stdout_write)(
+                (DELETE_CURSOR_IMAGES_b if now else DELETE_CURSOR_IMAGES)
+                if cursor
+                else (DELETE_ALL_IMAGES_b if now else DELETE_ALL_IMAGES)
+            )
 
-        arguments = locals()
-        super().draw(
-            *args,
-            **kwargs,
-            **{
-                var: arguments[var]
-                for var, default in __class__.draw.__kwdefaults__.items()
-                if arguments[var] is not default
-            },
-        )
+    def draw(self, *args, **kwargs):
+        # Ignore (and omit) native animation arguments for non-animations
+        if not (self._is_animated and kwargs.get("animate", True)):
+            for arg_name in ("native", "stall_native"):
+                try:
+                    del kwargs[arg_name]
+                except KeyError:
+                    pass
+
+        super().draw(*args, **kwargs)
 
     @classmethod
     def is_supported(cls):
@@ -347,12 +535,6 @@ class ITerm2Image(GraphicsImage):
         **kwargs,
     ):
         if native:
-            if self._TERM == "konsole":
-                raise _style_error(type(self))(
-                    "Native animation is not supported in the active terminal"
-                )
-            if img.format == "WEBP":
-                raise _style_error(type(self))("Native WEBP animation is not supported")
             try:
                 print(
                     self._format_render(
@@ -419,24 +601,26 @@ class ITerm2Image(GraphicsImage):
         is_on_konsole = self._TERM == "konsole"
         is_on_wezterm = self._TERM == "wezterm"
         jump_right = f"{CSI}{r_width}C"
+        jump_up = f"{CSI}{r_height - 1}A" if r_height > 1 else ""
         erase = f"{CSI}{r_width}X" if not mix and is_on_wezterm else ""
 
         file_is_readable = True
         if self._source_type is ImageSource.PIL_IMAGE:
             try:
-                open(img.filename)
+                file_is_readable = os.access(img.filename, os.R_OK)
             except (AttributeError, OSError):
                 file_is_readable = False
 
-        if native and self._is_animated and not frame and img.format != "WEBP":
+        if native and self._is_animated and not frame:
             if self._source_type is ImageSource.PIL_IMAGE:
                 if file_is_readable:
                     compressed_image = open(img.filename, "rb")
                 else:
+                    compressed_image = io.BytesIO()
                     try:
-                        compressed_image = io.BytesIO()
                         img.save(compressed_image, img.format, save_all=True)
                     except ValueError:
+                        self._close_image(img)
                         raise _style_error(type(self))(
                             "Native animation not supported: This image was sourced "
                             "from a PIL image of an unknown format"
@@ -444,12 +628,11 @@ class ITerm2Image(GraphicsImage):
             else:
                 compressed_image = open(self._source, "rb")
 
-            if img is not self._source:
-                img.close()
+            self._close_image(img)
 
             with compressed_image:
                 compressed_image.seek(0, 2)
-                if compressed_image.tell() > __class__.NATIVE_ANIM_MAXSIZE:
+                if compressed_image.tell() > self.native_anim_max_bytes:
                     warnings.warn(
                         "Image data size above the maximum for native animation",
                         TermImageWarning,
@@ -458,19 +641,26 @@ class ITerm2Image(GraphicsImage):
                 control_data = "".join(
                     (
                         f"size={compressed_image.tell()};width={r_width}"
-                        f";height={r_height};preserveAspectRatio=0;inline=1:"
+                        f";height={r_height};preserveAspectRatio=0;inline=1"
+                        f"{';doNotMoveCursor=1' * is_on_konsole}:"
                     )
                 )
                 compressed_image.seek(0)
                 return "".join(
                     (
-                        f"{erase}{jump_right}\n" * (r_height - 1),
+                        (
+                            ""
+                            if is_on_konsole
+                            else f"{erase}{jump_right}\n" * (r_height - 1)
+                        ),
                         erase,
-                        f"{CSI}{r_height - 1}A",
+                        "" if is_on_konsole else jump_up,
                         START,
                         control_data,
                         standard_b64encode(compressed_image.read()).decode(),
                         ST,
+                        f"{jump_right}\n" * (r_height - 1) if is_on_konsole else "",
+                        jump_right * is_on_konsole,
                     )
                 )
 
@@ -478,7 +668,7 @@ class ITerm2Image(GraphicsImage):
         width, height = self._get_minimal_render_size(adjust=render_method == LINES)
 
         if (  # Read directly from file when possible and reasonable
-            self.READ_FROM_FILE
+            self.read_from_file
             and not self._is_animated
             and file_is_readable
             and render_method == WHOLE
@@ -504,9 +694,9 @@ class ITerm2Image(GraphicsImage):
             img = self._get_render_data(
                 img, alpha, size=(width, height), pixel_data=False, frame=frame
             )[0]  # fmt: skip
-            if self.JPEG_QUALITY >= 0 and img.mode == "RGB":
+            if self.jpeg_quality >= 0 and img.mode == "RGB":
                 format = "jpeg"
-                jpeg_quality = min(self.JPEG_QUALITY, 95)
+                jpeg_quality = self.jpeg_quality
             else:
                 format = "png"
                 jpeg_quality = None
@@ -524,8 +714,8 @@ class ITerm2Image(GraphicsImage):
                 )
 
         # clean up (ImageIterator uses one PIL image throughout)
-        if frame_img is not img is not self._source:
-            img.close()
+        if frame_img is not img:
+            self._close_image(img)
 
         if render_method == LINES:
             # NOTE: It's more efficient to write separate strings to the buffer
@@ -579,7 +769,7 @@ class ITerm2Image(GraphicsImage):
                 (
                     "" if is_on_konsole else f"{erase}{jump_right}\n" * (r_height - 1),
                     erase,
-                    "" if is_on_konsole else f"{CSI}{r_height - 1}A",
+                    "" if is_on_konsole else jump_up,
                     START,
                     control_data,
                     standard_b64encode(compressed_image.read()).decode(),
@@ -591,6 +781,5 @@ class ITerm2Image(GraphicsImage):
 
 
 START = f"{OSC}1337;File="
-DELETE_ALL_IMAGES = f"{ESC}_Ga=d;{ST}".encode()
 native_anim = Event()
-_stdout_write = sys.stdout.buffer.write
+_stdout_write = sys.stdout.write
