@@ -12,7 +12,7 @@ from zlib import compress, decompress
 
 import PIL
 
-from ..utils import CSI, ESC, ST, get_terminal_name_version, query_terminal
+from ..utils import CSI, ESC, ST, get_terminal_name_version, query_terminal, write_tty
 from .common import GraphicsImage
 
 # Constants for render methods
@@ -25,6 +25,8 @@ class KittyImage(GraphicsImage):
 
     See :py:class:`GraphicsImage` for the complete description of the constructor.
 
+    |
+
     **Render Methods**
 
     :py:class:`KittyImage` provides two methods of :term:`rendering` images, namely:
@@ -35,8 +37,8 @@ class KittyImage(GraphicsImage):
 
        Pros:
 
-         * Good for use cases where it might be required to trim some lines of the
-           image.
+       * Good for use cases where it might be required to trim some lines of the
+         image.
 
     WHOLE
        Renders an image all at once i.e the entire image data is encoded into one
@@ -45,13 +47,53 @@ class KittyImage(GraphicsImage):
 
        Pros:
 
-         * Render results are more compact (i.e less in character count) than with
-           the **LINES** method since the entire image is encoded at once.
+       * Render results are more compact (i.e less in character count) than with
+         the **LINES** method since the entire image is encoded at once.
 
     The render method can be set with
     :py:meth:`set_render_method() <BaseImage.set_render_method>` using the names
     specified above.
 
+    |
+
+    **Style-Specific Render Parameters**
+
+    See :py:meth:`BaseImage.draw` (particularly the *style* parameter).
+
+    * **method** (*None | str*) → Render method override.
+
+      * ``None`` → the current effective render method of the instance is used.
+      * *default* → ``None``
+
+    * **z_index** (*int*) → The stacking order of graphics and text for
+      **non-animations**.
+
+      * An integer in the **signed 32-bit** range (excluding ``-(2**31)``)
+      * ``>= 0`` → the image will be drawn above text
+      * ``< 0`` → the image will be drawn below text
+      * ``< -(2**31)/2`` → the image will be drawn below cells with non-default
+        background color
+      * *default* → ``0``
+      * Overlapping graphics on different z-indexes will be blended (by the terminal
+        emulator) if they are semi-transparent.
+      * To inter-mix text with graphics, see the *mix* parameter.
+
+    * **mix** (*bool*) → Graphics/Text inter-mix policy.
+
+      * ``False`` → text within the region covered by the drawn render output will be
+        erased, though text can be inter-mixed with graphics after drawing
+      * ``True`` → text within the region covered by the drawn render output will NOT
+        be erased
+      * *default* → ``False``
+
+    * **compress** (*int*) → ZLIB compression level.
+
+      * ``0`` <= *compress* <= ``9``
+      * ``1`` → best speed, ``9`` → best compression, ``0`` → no compression
+      * *default* → ``4``
+      * Results in a trade-off between render time and data size/draw speed
+
+    |
 
     **Format Specification**
 
@@ -59,61 +101,63 @@ class KittyImage(GraphicsImage):
 
     ::
 
-        [method] [ z [index] ] [ m {0 | 1} ] [ c {0-9} ]
+        [ <method> ]  [ z <z-index> ]  [ m <mix> ]  [ c <compress> ]
 
-    * ``method``: Render method override.
+    * ``method`` → render method override
 
-      Can be one of:
+      * ``L`` → **LINES** render method (current frame only, for animated images)
+      * ``W`` → **WHOLE** render method (current frame only, for animated images)
+      * *default* → Current effective render method of the image
 
-        * ``L``: **LINES** render method (current frame only, for animated images).
-        * ``W``: **WHOLE** render method (current frame only, for animated images).
+    * ``z`` → graphics/text stacking order
 
-      Default: Current effective render method of the image.
+      * ``z-index`` → z-index
 
-    * ``z``: Image/Text stacking order.
+        * An integer in the **signed 32-bit** range (excluding ``-(2**31)``)
+        * ``>= 0`` → the render output will be drawn above text
+        * ``< 0`` → the render output will be drawn below text
+        * ``< -(2**31)/2`` → the render output will be drawn below cells with
+          non-default background color
 
-      * ``index``: Image z-index. An integer in the **signed 32-bit range**.
+      * *default* → ``z0`` (z-index zero)
+      * e.g ``z0``, ``z1``, ``z-1``, ``z2147483647``, ``z-2147483648``
+      * overlapping graphics on different z-indexes will be blended
+        (by the terminal emulator) if they are semi-transparent
 
-        Images drawn in the same location with different z-index values will be
-        blended if they are semi-transparent. If ``index`` is:
+    * ``m`` → graphics/text inter-mix policy
 
-        * ``>= 0``, the image will be drawn above text.
-        * ``< 0``, the image will be drawn below text.
-        * ``< -(2**31)/2``, the image will be drawn below cells with non-default
-          background color.
+      * ``mix`` → inter-mix policy
 
-      * ``z`` without ``index`` is currently only used internally.
-      * If *absent*, defaults to ``z0`` i.e z-index zero.
-      * e.g ``z0``, ``z1``, ``z-1``, ``z2147483647``, ``z-2147483648``.
+        * ``0`` → text within the region covered by the drawn render output will be
+          erased, though text can be inter-mixed with graphics after drawing
+        * ``1`` → text within the region covered by the drawn render output will NOT
+          be erased
 
-    * ``m``: Image/Text inter-mixing policy.
+      * *default* → ``m0``
+      * e.g ``m0``, ``m1``
 
-      * If the character after ``m`` is:
+    * ``c`` → ZLIB compression level
 
-        * ``0``, text within the region covered by the image will be erased,
-          though text can be inter-mixed with the image after it's been drawn.
-        * ``1``, text within the region covered by the image will NOT be erased.
+      * ``compress`` → compression level
 
-      * If *absent*, defaults to ``m0``.
-      * e.g ``m0``, ``m1``.
+        * An integer in the range ``0`` <= ``compress`` <= ``9``
+        * ``1`` → best speed, ``9`` → best compression, ``0`` → no compression
 
-    * ``c``: ZLIB compression level.
+      * *default* → ``c4``
+      * e.g ``c0``, ``c9``
+      * results in a trade-off between render time and data size/draw speed
 
-      * 1 -> best speed, 9 -> best compression, 0 -> no compression.
-      * This results in a trade-off between render time and data size/draw speed.
-      * If *absent*, defaults to ``c4``.
-      * e.g ``c0``, ``c9``.
+    |
 
+    IMPORTANT:
+        Currently supported terminal emulators are:
 
-    ATTENTION:
-        Currently supported terminal emulators include:
-
-          * `Kitty <https://sw.kovidgoyal.net/kitty/>`_ >= 0.20.0.
-          * `Konsole <https://konsole.kde.org>`_ >= 22.04.0.
+        * `Kitty <https://sw.kovidgoyal.net/kitty/>`_ >= 0.20.0.
+        * `Konsole <https://konsole.kde.org>`_ >= 22.04.0.
     """
 
     _FORMAT_SPEC: Tuple[re.Pattern] = tuple(
-        map(re.compile, r"[LW] z(-?\d+)? m[01] c[0-9]".split(" "))
+        map(re.compile, r"[LW] z-?\d+ m[01] c[0-9]".split(" "))
     )
     _render_methods: Set[str] = {LINES, WHOLE}
     _default_render_method: str = LINES
@@ -133,19 +177,21 @@ class KittyImage(GraphicsImage):
         "z_index": (
             0,
             (
-                lambda x: x is None or isinstance(x, int),
-                "z-index must be `None` or an integer",
+                lambda x: isinstance(x, int),
+                "z-index must be an integer",
             ),
             (
-                lambda x: x is None or -(2**31) <= x < 2**31,
-                "z-index must be within the 32-bit signed integer range",
+                # INT32_MIN is reserved for non-native animations
+                lambda x: -(2**31) < x < 2**31,
+                "z-index must be within the 32-bit signed integer range "
+                "(excluding ``-(2**31)``)",
             ),
         ),
         "mix": (
             False,
             (
                 lambda x: isinstance(x, bool),
-                "Inter-mixing policy must be a boolean",
+                "Inter-mix policy must be a boolean",
             ),
             (lambda _: True, ""),
         ),
@@ -166,82 +212,79 @@ class KittyImage(GraphicsImage):
     _TERM_VERSION: str = ""
     _KITTY_VERSION: Tuple[int, int, int] = ()
 
-    @staticmethod
-    def clear(all: bool = True) -> None:
-        """Clears images on-screen.
-
-        Args:
-            all: If ``False``, clears only the images intersecting with the cursor.
-              Otherwise, clears all images currently on the screen.
-        """
-        _stdout_write(DELETE_ALL_IMAGES if all else DELETE_CURSOR_IMAGES)
-
-    # Only defined for the purpose of proper self-documentation
-    def draw(
-        self,
-        *args,
-        method: Optional[str] = None,
-        z_index: Optional[int] = 0,
-        mix: bool = False,
-        compress: int = 4,
-        **kwargs,
+    @classmethod
+    def clear(
+        cls, *, cursor: bool = False, z_index: Optional[int] = None, now: bool = False
     ) -> None:
-        """Draws an image to standard output.
-
-        Extends the common interface with style-specific parameters.
+        """Clears images.
 
         Args:
-            args: Positional arguments passed up the inheritance chain.
-            method: Render method override. If ``None`` or not given, the current
-              effective render method of the instance is used.
-            z_index: The stacking order of images and text **for non-animations**.
+            cursor: If ``True``, all images intersecting with the current cursor
+              position are cleared.
+            z_index: An integer in the **signed 32-bit range**. If given, all images
+              on the given z-index are cleared.
+            now: If ``True`` the images are cleared immediately, without affecting
+              any standard I/O stream.
+              Otherwise they're cleared when next :py:data:`sys.stdout` is flushed.
 
-              Images drawn in the same location with different z-index values will be
-              blended if they are semi-transparent. If *z_index* is:
+        Aside *now*, **only one** other argument may be given. If no argument is given
+        (aside *now*) or default values are given, all images visible on the screen are
+        cleared.
 
-              * ``>= 0``, the image will be drawn above text.
-              * ``< 0``, the image will be drawn below text.
-              * ``< -(2**31)/2``, the image will be drawn below cells with
-                non-default background color.
-              * ``None``, internal use only, mentioned for the sake of completeness.
-
-              To inter-mixing text with an image, see the *mix* parameter.
-
-            mix: Image/Text inter-mixing policy **for non-animations**. If:
-
-              * ``True``, text within the region covered by the image will NOT be
-                erased.
-              * ``False``, text within the region covered by the image will be
-                erased, though text can be inter-mixed with the image after it's
-                been drawn.
-
-            compress: ZLIB compression level.
-
-              An integer between 0 and 9: 1 -> best speed, 9 -> best compression, 0 ->
-              no compression. This results in a trade-off between render time and data
-              size/draw speed.
-
-            kwargs: Keyword arguments passed up the inheritance chain.
-
-        See the ``draw()`` method of the parent classes for full details, including the
-        description of other parameters.
+        NOTE:
+            This method does nothing if the render style is not supported.
         """
-        arguments = locals()
-        super().draw(
-            *args,
-            **kwargs,
-            **{
-                var: arguments[var]
-                for var, default in __class__.draw.__kwdefaults__.items()
-                if arguments[var] is not default
-            },
-        )
+        if not (cls._forced_support or cls.is_supported()):
+            return
+
+        if not isinstance(cursor, bool):
+            raise TypeError(f"Invalid type for 'cursor' (got: {type(cursor).__name__})")
+
+        if z_index is not None:
+            if not isinstance(z_index, int):
+                raise TypeError(
+                    f"Invalid type for 'z_index' (got: {type(z_index).__name__})"
+                )
+            if not -(1 << 31) <= z_index < (1 << 31):
+                raise ValueError(
+                    "z-index must be within the 32-bit signed integer range "
+                    f"(got: {z_index})"
+                )
+
+        if not isinstance(now, bool):
+            raise TypeError(f"Invalid type for 'now' (got: {type(now).__name__})")
+
+        default_args = __class__.clear.__func__.__kwdefaults__
+        nonlocals = locals()
+        args = {name: nonlocals[name] for name in default_args}
+        given_args = args.items() - (default_args.items() | {("now", True)})
+
+        if len(given_args) > 1:
+            raise ValueError("Only one argument may be given")
+        elif given_args:
+            arg, _ = given_args.pop()
+            (write_tty if now else _stdout_write)(
+                (DELETE_CURSOR_IMAGES_b if now else DELETE_CURSOR_IMAGES)
+                if arg == "cursor"
+                else (
+                    (DELETE_Z_INDEX_IMAGES_b if now else DELETE_Z_INDEX_IMAGES)
+                    % z_index
+                )
+            )
+        elif now:
+            write_tty(DELETE_ALL_IMAGES_b)
+        else:
+            _stdout_write(DELETE_ALL_IMAGES)
 
     @classmethod
-    def is_supported(cls):
+    def is_supported(cls) -> bool:
         if cls._supported is None:
             cls._supported = False
 
+            # The graphics query for support detection messes up iTerm2's window title
+            if get_terminal_name_version()[0] == "iterm2":
+                return False
+
             # Kitty graphics query + terminal attribute query
             # The second query is to speed up the query since most (if not all)
             # terminals should support it and most terminals treat queries as FIFO
@@ -278,7 +321,7 @@ class KittyImage(GraphicsImage):
 
     @classmethod
     def _check_style_format_spec(cls, spec: str, original: str) -> Dict[str, Any]:
-        parent, (method, (z, index), mix, compress) = cls._get_style_format_spec(
+        parent, (method, z_index, mix, compress) = cls._get_style_format_spec(
             spec, original
         )
         args = {}
@@ -286,8 +329,8 @@ class KittyImage(GraphicsImage):
             args.update(super()._check_style_format_spec(parent, original))
         if method:
             args["method"] = LINES if method == "L" else WHOLE
-        if z:
-            args["z_index"] = index and int(index)
+        if z_index:
+            args["z_index"] = int(z_index[1:])
         if mix:
             args["mix"] = bool(int(mix[-1]))
         if compress:
@@ -299,26 +342,22 @@ class KittyImage(GraphicsImage):
     def _clear_frame(cls) -> bool:
         """Clears an animation frame on-screen.
 
-        | Only used on Kitty <= 0.25.0 because ``z_index=None`` is buggy on these
+        | Only used on Kitty <= 0.25.0 because ``blend=False`` is buggy on these
           versions. Does nothing on any other version or terminal.
-        | Note that this implementation might do more than required since it clears
-          all images on screen.
+        | Also clears any frame of any previously drawn animation, since they all use
+          the same z-index.
 
         See :py:meth:`~term_image.image.BaseImage._clear_frame` for description.
         """
         if cls._KITTY_VERSION and cls._KITTY_VERSION <= (0, 25, 0):
-            cls.clear()
+            cls.clear(z_index=-(1 << 31))
             return True
         return False
 
     def _display_animated(self, *args, **kwargs) -> None:
+        kwargs["z_index"] = -(1 << 31)
         if self._KITTY_VERSION > (0, 25, 0):
-            kwargs["z_index"] = None
-        else:
-            try:
-                del kwargs["z_index"]
-            except KeyError:
-                pass
+            kwargs["blend"] = False
 
         super()._display_animated(*args, **kwargs)
 
@@ -348,10 +387,19 @@ class KittyImage(GraphicsImage):
         *,
         frame: bool = False,
         method: Optional[str] = None,
-        z_index: Optional[int] = 0,
+        z_index: int = 0,
         mix: bool = False,
         compress: int = 4,
+        blend: bool = True,
     ) -> str:
+        """See :py:meth:`BaseImage._render_image` for the description of the method and
+        :py:meth:`draw` for parameters not described here.
+
+        Args:
+            blend: If ``False``, the rendered image deletes overlapping/intersecting
+              images when drawn. Otherwise, the behaviour is dependent on the z-index
+              and/or the terminal emulator (for images with the same z-index).
+        """
         # NOTE: It's more efficient to write separate strings to the buffer separately
         # than concatenate and write together.
 
@@ -372,13 +420,13 @@ class KittyImage(GraphicsImage):
         raw_image = img.tobytes()
 
         # clean up (ImageIterator uses one PIL image throughout)
-        if frame_img is not img is not self._source:
-            img.close()
+        if frame_img is not img:
+            self._close_image(img)
 
         control_data = ControlData(f=format, s=width, c=r_width, z=z_index)
         erase = "" if mix else f"{CSI}{r_width}X"
         jump_right = f"{CSI}{r_width}C"
-        if z_index is None:
+        if not blend:
             delete = f"{START}a=d,d=C;{ST}"
 
         if render_method == LINES:
@@ -390,7 +438,7 @@ class KittyImage(GraphicsImage):
                 trans = Transmission(
                     control_data, raw_image.read(bytes_per_line), compress
                 )
-                z_index is None and buffer.write(delete)
+                blend or buffer.write(delete)
                 for chunk in trans.get_chunks():
                     buffer.write(chunk)
                 # Writing spaces clears any text under transparent areas of an image
@@ -401,7 +449,7 @@ class KittyImage(GraphicsImage):
                     trans = Transmission(
                         control_data, raw_image.read(bytes_per_line), compress
                     )
-                    z_index is None and buffer.write(delete)
+                    blend or buffer.write(delete)
                     for chunk in trans.get_chunks():
                         buffer.write(chunk)
                 buffer.write(erase)
@@ -412,7 +460,7 @@ class KittyImage(GraphicsImage):
         vars(control_data).update(v=height, r=r_height)
         return "".join(
             (
-                z_index is None and delete or "",
+                ("" if blend else delete),
                 Transmission(control_data, raw_image, compress).get_chunked(),
                 f"{erase}{jump_right}\n" * (r_height - 1),
                 f"{erase}{jump_right}",
@@ -577,6 +625,10 @@ class _ControlData:  # Currently Unused
 
 START = f"{ESC}_G"
 FMT = f"{START}%(control)s;%(payload)s{ST}"
-DELETE_ALL_IMAGES = f"{ESC}_Ga=d;{ST}".encode()
-DELETE_CURSOR_IMAGES = f"{ESC}_Ga=d,d=C;{ST}".encode()
-_stdout_write = sys.stdout.buffer.write
+DELETE_ALL_IMAGES = f"{ESC}_Ga=d,d=A;{ST}"
+DELETE_ALL_IMAGES_b = DELETE_ALL_IMAGES.encode()
+DELETE_CURSOR_IMAGES = f"{ESC}_Ga=d,d=C;{ST}"
+DELETE_CURSOR_IMAGES_b = DELETE_CURSOR_IMAGES.encode()
+DELETE_Z_INDEX_IMAGES = f"{ESC}_Ga=d,d=Z,z=%d;{ST}"
+DELETE_Z_INDEX_IMAGES_b = DELETE_Z_INDEX_IMAGES.encode()
+_stdout_write = sys.stdout.write
