@@ -1,25 +1,14 @@
 """
-Utilities
-=========
+.. Utilities
 """
 
 from __future__ import annotations
 
 __all__ = (
-    "OS_IS_UNIX",
-    "no_redecorate",
-    "cached",
-    "lock_tty",
-    "unix_tty_only",
-    "terminal_size_cached",
-    "color",
-    "get_cell_size",
-    "get_fg_bg_colors",
+    "get_terminal_name_version",
     "get_terminal_size",
-    "get_window_size",
-    "query_terminal",
-    "read_tty",
-    "set_query_timeout",
+    "lock_tty",
+    "read_tty_all",
     "write_tty",
 )
 
@@ -31,7 +20,6 @@ from array import array
 from functools import wraps
 from multiprocessing import Array, Process, Queue as mp_Queue, RLock as mp_RLock
 from operator import floordiv
-from pathlib import Path
 from queue import Empty, Queue
 from shutil import get_terminal_size as _get_terminal_size
 from threading import RLock
@@ -53,16 +41,8 @@ except ImportError:
 else:
     OS_IS_UNIX = True
 
-#: If ``True``, :ref:`terminal-queries` are disabled, thereby affecting all
-#: :ref:`dependent features <queried-features>`.
-DISABLE_QUERIES: bool = False
-
-#: A workaround for some terminal emulators (e.g older VTE-based ones) that wrongly
-#: report window dimensions swapped.
-#:
-#: | If ``True``, the dimensions reported by the terminal emulator are swapped.
-#: | This setting affects :ref:`auto-cell-ratio` computation.
-SWAP_WIN_SIZE: bool = False
+# NOTE: Any cached feature using a query should have it's cache invalidated in
+# `term_image.enable_queries()`.
 
 # Decorator Classes
 
@@ -72,16 +52,104 @@ class ClassInstanceMethod(classmethod):
     and when invoked via an instance, behaves like an instance method.
     """
 
-    def __get__(self, obj, cls=None):
-        # classmethod just uses cls directly if present.
-        # Otherwise, type(obj) but we're not concerned with that.
-        return super().__get__(None, obj or cls)
+    def __get__(self, instance, owner=None):
+        # classmethod just uses `owner` directly if given.
+        # Otherwise, type(instance) but we're not concerned with this.
+        return super().__get__(None, instance or owner)
+
+
+class ClassPropertyBase(property):
+    """Base class for properties that operate on their **owner**.
+
+    NOTE:
+        For ``__set__()`` and ``__delete__()`` to actually work, the metaclass of the
+        owner must be made to recognize and directly call these methods within it's
+        ``__setattr__()`` and ``__delattr__()``.
+
+        If the owner defines (not inherits) a ``_class_properties_`` attribute, it must
+        be a :py:class:``dict`` and is populated with the names (as in the owner's
+        namespace) of all instances of this class (i.e class properties) mapped to the
+        respective instances.
+
+        For this reason, :py:class:`ClassPropertyMeta` is provided.
+    """
+
+    def __init__(self, fget=None, fset=None, fdel=None, doc=None):
+        super().__init__(fget, fset, fdel, doc)
+        # `property` doesn't set `__doc__`, probably cos this class' `__doc__`
+        # attribute overrides its `__doc__` descriptor.
+        super().__setattr__("__doc__", doc)
+
+    def __set_name__(self, owner, name):
+        self.__objclass__ = owner
+        try:
+            class_properties = owner.__dict__["_class_properties_"]
+        except KeyError:
+            pass
+        else:
+            class_properties[name] = self
+
+
+class ClassInstanceProperty(ClassPropertyBase):
+    """A property which operates on the invoker, be it the owner or an instance."""
+
+    def __get__(self, instance, owner=None):
+        return super().__get__(instance or owner, owner)
+
+
+class ClassProperty(ClassPropertyBase):
+    """A property which always operates on the owner, even when invoked by an
+    instance.
+    """
+
+    def __get__(self, instance, owner=None):
+        return super().__get__(owner, owner)
+
+    def __set__(self, obj, value):
+        super().__set__(type(obj) if isinstance(obj, self.__objclass__) else obj, value)
+
+    def __delete__(self, obj):
+        super().__delete__(type(obj) if isinstance(obj, self.__objclass__) else obj)
+
+
+class ClassPropertyMeta(type):
+    """A metaclass that implements support for modifiable class properties owned by
+    its instances.
+
+    - Takes care of inherited class properties.
+    - Works with both cooperative multiple ane multi-level inheritance.
+
+    SEE ALSO:
+        :py:class:`ClassPropertyBase`
+    """
+
+    def __new__(cls, name, bases, dict, **kwds):
+        class_properties = dict["_class_properties_"] = {}
+        for base in bases:
+            if isinstance(base, __class__):
+                class_properties.update(base._class_properties_)
+
+        return super().__new__(cls, name, bases, dict, **kwds)
+
+    def __setattr__(self, name, value):
+        class_property = self._class_properties_.get(name)
+        if class_property:
+            class_property.__set__(self, value)
+        else:
+            super().__setattr__(name, value)
+
+    def __delattr__(self, name):
+        class_property = self._class_properties_.get(name)
+        if class_property:
+            class_property.__delete__(self)
+        else:
+            super().__delattr__(name)
 
 
 # Decorator Functions
 
 
-def no_redecorate(decor: Callable) -> FunctionType:
+def no_redecorate(decor: FunctionType) -> FunctionType:
     """Prevents a decorator from re-decorating objects.
 
     Args:
@@ -102,7 +170,7 @@ def no_redecorate(decor: Callable) -> FunctionType:
 
 
 @no_redecorate
-def cached(func: Callable) -> FunctionType:
+def cached(func: FunctionType) -> FunctionType:
     """Enables return value caching.
 
     Args:
@@ -140,25 +208,25 @@ def cached(func: Callable) -> FunctionType:
 
 
 @no_redecorate
-def lock_tty(func: Callable) -> FunctionType:
+def lock_tty(func: FunctionType) -> FunctionType:
     """Synchronizes access to the :term:`active terminal`.
 
     Args:
         func: The function to be wrapped.
 
-    When any decorated function is called, a re-entrant lock is acquired by the current
+    When a decorated function is called, a re-entrant lock is acquired by the current
     process or thread and released after the call, such that any other decorated
     function called within another thread or subprocess has to wait till the lock is
     fully released (i.e has been released as many times as acquired) by the current
     process or thread.
 
     NOTE:
-        | It automatically works across parent-/sub-processes, started directly or
-          indirectly via ``multiprocessing.Process`` (or a subclass of it) and their
-          threads.
+        It automatically works across parent-/sub-processes, started directly or
+        indirectly via :py:class:`multiprocessing.Process` (or a subclass of it) and
+        their threads.
 
     IMPORTANT:
-        It only works if ``multiprocessing.synchronize`` is supported on the host
+        It only works if :py:mod:`multiprocessing.synchronize` is supported on the host
         platform.  If not supported, a warning is issued when starting a subprocess.
     """
 
@@ -168,11 +236,21 @@ def lock_tty(func: Callable) -> FunctionType:
             # logging.debug(f"{func.__name__} acquired TTY lock", stacklevel=3)
             return func(*args, **kwargs)
 
+    if func.__doc__ is not None:
+        lock_tty_wrapper.__doc__ = (
+            func.__doc__.rstrip()
+            + """
+
+        IMPORTANT:
+            Synchronized with :py:func:`~term_image.utils.lock_tty`.
+        """
+        )
+
     return lock_tty_wrapper
 
 
 @no_redecorate
-def terminal_size_cached(func: Callable) -> FunctionType:
+def terminal_size_cached(func: FunctionType) -> FunctionType:
     """Enables return value caching based on the size of the :term:`active terminal`.
 
     Args:
@@ -211,7 +289,7 @@ def terminal_size_cached(func: Callable) -> FunctionType:
 
 
 @no_redecorate
-def unix_tty_only(func: Callable) -> FunctionType:
+def unix_tty_only(func: FunctionType) -> FunctionType:
     """Disable invokation of a function on a non-unix-like platform or when there is no
     :term:`active terminal`.
 
@@ -303,9 +381,9 @@ def get_fg_bg_colors(
             # sequence, hence the repetition of OSC ... ST
             f"{OSC}10;?{ST}{OSC}11;?{ST}{CSI}c".encode(),
             # The response might contain a "c"; can't stop reading at "c"
-            lambda s: not s.endswith(CSI.encode()),
+            lambda s: not s.endswith(CSI_b),
         )
-        if not DISABLE_QUERIES:
+        if _queries_enabled:
             read_tty()  # The rest of the response to `CSI c`
 
     fg = bg = None
@@ -323,7 +401,12 @@ def get_fg_bg_colors(
 
 @cached
 def get_terminal_name_version() -> Tuple[Optional[str], Optional[str]]:
-    """Queries the :term:`active terminal` for it's name and version"""
+    """Returns the name and version of the :term:`active terminal`, if available.
+
+    Returns:
+        A 2-tuple, ``(name, version)``. If either is not available, returns ``None``
+        in its place.
+    """
     with _tty_lock:  # the terminal's response to the query is not read all at once
         # Terminal name/version query + terminal attribute query
         # The latter is to speed up the entire query since most (if not all)
@@ -331,9 +414,9 @@ def get_terminal_name_version() -> Tuple[Optional[str], Optional[str]]:
         response = query_terminal(
             f"{CSI}>q{CSI}c".encode(),
             # The response might contain a "c"; can't stop reading at "c"
-            lambda s: not s.endswith(CSI.encode()),
+            lambda s: not s.endswith(CSI_b),
         )
-        if not DISABLE_QUERIES:
+        if _queries_enabled:
             read_tty()  # The rest of the response to `CSI c`
 
     match = response and NAME_VERSION.fullmatch(response.decode().rpartition(ESC)[0])
@@ -352,11 +435,14 @@ def get_terminal_size() -> os.terminal_size:
     Returns:
         The terminal size in columns and lines.
 
-    Tries to query the :term:`active terminal` device and falls back to
-    ``shutil.get_terminal_size()`` if that fails.
+    NOTE:
+        This implementation is quite different from :py:func:`shutil.get_terminal_size`
+        and :py:func:`os.get_terminal_size` in that it:
 
-    This implementation still gives the correct size of the process' controlling
-    terminal when output is redirected (in most cases), unlike the fallback.
+        - gives the correct size of the :term:`active terminal` even when output is
+          redirected, in most cases
+        - gives different results in certain situations
+        - is what this library works with
     """
     if _tty:
         # faster and gives correct results when output is redirected
@@ -418,39 +504,11 @@ def get_window_size() -> Optional[Tuple[int, int]]:
                 if os.environ.get("SHELL", "").startswith("/data/data/com.termux/"):
                     size = (size[0], size[1] * 2)
 
-        size = size[:: -SWAP_WIN_SIZE or 1] if size else (0, 0)
+        size = size[:: -_swap_win_size or 1] if size else (0, 0)
         _win_size_cache[:] = ts + size
         return None if 0 in size else size
 
 
-def is_writable(path: Union[str, os.PathLike, Path]) -> bool:
-    """Checks if a file path is writable or creatable.
-
-    Returns:
-      - ``True``, if:
-        - the file exists and is writable
-        - the file doesn't exists but can be created
-      - ``False``, if:
-        - the path points to a directory
-        - the file exists but is unwritable
-        - the file doesn't exists and cannot be created
-    """
-    path = Path(path).expanduser()
-    writable = False
-
-    if path.exists():
-        if path.is_file() and os.access(path, os.W_OK):
-            writable = True
-    else:
-        for path in path.parents:
-            if path.exists():
-                if path.is_dir() and os.access(path, os.W_OK):
-                    writable = True
-                break
-
-    return writable
-
-
 @unix_tty_only
 @lock_tty
 def query_terminal(
@@ -468,18 +526,20 @@ def query_terminal(
         timeout: Time limit for awaiting a response from the terminal, in seconds
           (infinite if negative).
 
-          If not given or ``None``, :py:data:`QUERY_TIMEOUT` (set by
-          :py:func:`set_query_timeout`) is used.
+          If not given or ``None``, the value set by
+          :py:func:`~term_image.set_query_timeout`
+          (or :py:data:`~term_image.DEFAULT_QUERY_TIMEOUT` if never set) is used.
 
     Returns:
-        `None` if :py:data:`DISABLE_QUERIES` is true, else the terminal's response
-        (empty, if no response is recieved after *timeout* is up).
+        `None` if queries are disabled (via :py:func:`~term_image.disable_queries`),
+        else the terminal's response (empty, if no response is recieved after
+        *timeout* is up).
 
     ATTENTION:
         Any unread input is discared before the query. If the input might be needed,
         it can be read using :py:func:`read_tty()` before calling this fucntion.
     """
-    if DISABLE_QUERIES:
+    if not _queries_enabled:
         return None
 
     old_attr = termios.tcgetattr(_tty)
@@ -488,7 +548,7 @@ def query_terminal(
     try:
         termios.tcsetattr(_tty, termios.TCSAFLUSH, new_attr)
         write_tty(request)
-        return read_tty(more, timeout or QUERY_TIMEOUT)
+        return read_tty(more, timeout or _query_timeout)
     finally:
         termios.tcsetattr(_tty, termios.TCSANOW, old_attr)
 
@@ -501,7 +561,7 @@ def read_tty(
     min: int = 0,
     *,
     echo: bool = False,
-) -> Optional[bytes]:
+) -> bytes:
     """Reads input directly from the :term:`active terminal` with/without blocking.
 
     Args:
@@ -537,24 +597,6 @@ def read_tty(
     Upon return or interruption, the :term:`active terminal` is **immediately** restored
     to the state in which it was met.
     """
-    if not callable(more):
-        raise TypeError("'more' must be callable")
-    try:
-        ret = more(bytearray())
-    except Exception:
-        raise ValueError(
-            "'more' must be able to accept a single argument, a `bytearray` object"
-        ) from None
-    else:
-        if not isinstance(ret, bool):
-            raise TypeError("'more' must return a boolean")
-    if not isinstance(timeout, (type(None), float)):
-        raise TypeError("'timeout' must be `None` or a float")
-    if not isinstance(min, int):
-        raise TypeError("'min' must be an integer")
-    if not isinstance(echo, bool):
-        raise TypeError("'echo' must be a boolean")
-
     old_attr = termios.tcgetattr(_tty)
     new_attr = termios.tcgetattr(_tty)
     new_attr[3] &= ~termios.ICANON  # Disable cannonical mode
@@ -598,31 +640,27 @@ def read_tty(
     return bytes(input)
 
 
-def set_query_timeout(timeout: float) -> None:
-    """Sets the global timeout for :ref:`terminal-queries`.
+@unix_tty_only
+def read_tty_all() -> bytes:
+    """Reads all available input directly from the :term:`active terminal` **without
+    blocking**.
 
-    Args:
-        timeout: Time limit for awaiting a response from the terminal, in seconds.
+    Returns:
+        The input read.
 
-    Raises:
-        TypeError: *timeout* is not a float.
-        ValueError: *timeout* is less than or equal to zero.
+    HINT:
+        Synchronized with :py:func:`lock_tty`.
     """
-    global QUERY_TIMEOUT
-
-    if not isinstance(timeout, float):
-        raise TypeError(f"'timeout' must be a float (got: {type(timeout).__name__!r})")
-    if timeout <= 0.0:
-        raise ValueError(f"'timeout' must be greater than zero (got: {timeout!r})")
-
-    QUERY_TIMEOUT = timeout
+    return read_tty()
 
 
 @unix_tty_only
 @lock_tty
 def write_tty(data: bytes) -> None:
-    """Writes *data* to the :term:`active terminal` and waits until it's completely
-    transmitted.
+    """Writes to the :term:`active terminal` and waits until complete transmission.
+
+    Args:
+        data: Data to be written.
     """
     os.write(_tty, data)
     try:
@@ -655,10 +693,10 @@ def _process_start_wrapper(self, *args, **kwargs):
                     "Hence, if any subprocess will be writing/reading to/from the "
                     "active terminal, it may be unsafe to use any features requiring"
                     "terminal queries.\n"
-                    "See https://term-image.readthedocs.io/en/stable/library/reference"
-                    "/utils.html#terminal-queries\n"
-                    "If any related issues occur, it's advisable to set "
-                    "`term_image.utils.DISABLE_QUERIES = True`.\n"
+                    "See https://term-image.readthedocs.io/en/stable/guide/concepts"
+                    ".html#terminal-queries\n"
+                    "If any related issues occur, it's advisable to disable queries "
+                    "using `term_image.disable_queries()`.\n"
                     "Simply set an 'ignore' filter for this warning (before starting "
                     "any subprocess) if not using any of the affected features.",
                     TermImageWarning,
@@ -692,21 +730,41 @@ def _process_run_wrapper(self, *args, set_tty_lock: bool = True, **kwargs):
     return _process_run_wrapper.__wrapped__(self, *args, **kwargs)
 
 
-QUERY_TIMEOUT = 0.1
 RGB_SPEC = re.compile(r"\033](\d+);rgb:([\da-fA-F/]+)\033\\", re.ASCII)
 WIN_SIZE = re.compile(r"\033\[4;(\d+);(\d+)t", re.ASCII)
 NAME_VERSION = re.compile(r"\033P>\|(\w+)[( ]([^)\033]+)\)?\033\\", re.ASCII)
 
 # Constants for escape sequences
-
 ESC = "\033"
+ESC_b = ESC.encode()
 CSI = f"{ESC}["
+CSI_b = CSI.encode()
 OSC = f"{ESC}]"
+OSC_b = OSC.encode()
 ST = f"{ESC}\\"
+ST_b = ST.encode()
 BG_FMT = f"{CSI}48;2;%d;%d;%dm"
+BG_FMT_b = BG_FMT.encode()
 FG_FMT = f"{CSI}38;2;%d;%d;%dm"
+FG_FMT_b = FG_FMT.encode()
 COLOR_RESET = f"{CSI}m"
+COLOR_RESET_b = COLOR_RESET.encode()
+DECSET = f"{CSI}?%dh"
+DECSET_b = DECSET.encode()
+DECRST = f"{CSI}?%dl"
+DECRST_b = DECRST.encode()
 
+# Terminal Synchronized Output
+# See https://gist.github.com/christianparpart/d8a62cc1ab659194337d73e399004036
+BEGIN_SYNCED_UPDATE = DECSET % 2026
+BEGIN_SYNCED_UPDATE_b = BEGIN_SYNCED_UPDATE.encode()
+END_SYNCED_UPDATE = DECRST % 2026
+END_SYNCED_UPDATE_b = END_SYNCED_UPDATE.encode()
+
+# Private internal variables
+_query_timeout = 0.1
+_queries_enabled: bool = True
+_swap_win_size: bool = False
 _tty: Optional[int] = None
 _tty_lock = RLock()
 _win_size_cache = [0] * 4
@@ -726,10 +784,11 @@ if OS_IS_UNIX:
             warnings.warn(
                 "It seems this process is not running within a terminal. "
                 "Hence, some features will behave differently or be disabled.\n"
-                "See https://term-image.readthedocs.io/en/stable/library/reference"
-                "/utils.html#terminal-queries\n"
-                "You can set an 'ignore' filter for this warning before loading "
-                "`term_image`, if not using any of the features affected.",
+                "See https://term-image.readthedocs.io/en/stable/guide/concepts"
+                ".html#active-terminal\n"
+                "Any filter for this warning must be set before loading `term_image`, "
+                "using `UserWarning` with the warning message (since "
+                "`TermImageWarning` won't be available).",
                 TermImageWarning,
             )
     if _tty:
